<?php

namespace Drupal\sms_engagespark\Plugin\SmsGateway;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\sms\Message\SmsDeliveryReport;
use Drupal\sms\Message\SmsMessageInterface;
use Drupal\sms\Plugin\SmsGatewayPluginBase;
use Drupal\sms_engagespark\Sms;
use GuzzleHttp\ClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Implements engageSPARK gateway.
 *
 * @SmsGateway(
 *   id = "engagespark",
 *   label = @Translation("engageSPARK"),
 * )
 */
class EngageSpark extends SmsGatewayPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $client;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new engageSPARK plugin.
   *
   * @param array $configuration
   *   The configuration to use and build the sms gateway.
   * @param string $plugin_id
   *   The gateway id.
   * @param mixed $plugin_definition
   *   The gateway plugin definition.
   * @param \GuzzleHttp\ClientInterface $client
   *   The HTTP client.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ClientInterface $client, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->client = $client;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('http_client'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'from' => 'engageSPARK',
      'org_id' => NULL,
      'token' => NULL,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['sms'] = [
      '#type' => 'details',
      '#title' => $this->t('engageSPARK'),
      '#open' => TRUE,
    ];
    $form['sms']['org_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Organization ID'),
      '#default_value' => $this->configuration['org_id'],
      '#required' => TRUE,
    ];
    $form['sms']['from'] = [
      '#type' => 'textfield',
      '#title' => $this->t('From'),
      '#default_value' => $this->configuration['from'],
      '#required' => TRUE,
    ];
    $form['sms']['token'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Token'),
      '#default_value' => $this->configuration['token'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    // Save the sanitized credentials.
    $this->configuration['org_id'] = Xss::filter($form_state->getValue('org_id'));
    $this->configuration['from'] = Xss::filter($form_state->getValue('from'));
    $this->configuration['token'] = Xss::filter($form_state->getValue('token'));
  }

  /**
   * {@inheritdoc}
   */
  public function send(SmsMessageInterface $sms_message) {
    /** @var \Drupal\sms\Message\SmsDeliveryReport $message_report */
    $message_report = new SmsDeliveryReport();

    $sender = $this->configuration['from'];

    // Get the message plain text.
    $message = $sms_message->getMessage();

    // Get the Unix timestamp.
    $timestamp = \time();

    // Load SMS gateway entity.
    /** @var \Drupal\sms\Entity\SmsGatewayInterface $sms_gateway */
    $sms_gateway = $this->entityTypeManager
      ->getStorage('sms_gateway')
      ->load($this->getPluginId());

    $sms_message->setSender($sender);
    $sms_message->setSenderNumber($sender);
    $sms_message->setGateway($sms_gateway);

    foreach ($sms_message->getRecipients() as $recipient) {
      $sms = new Sms();
      $response = $sms
        ->setClient($this->client)
        ->setCredentials([
          'token' => $this->configuration['token'],
        ])
        ->setOrgId($this->configuration['org_id'])
        ->setSender($sender)
        ->setRecipient($recipient)
        ->setMessage($message)
        ->send();

      if ($response) {
        $message_report->setStatusMessage($message);
        $message_report->setStatusTime($timestamp);
        $message_report->setRecipient($recipient);
      }

      return $response;
    }
  }

}
