<?php

namespace Drupal\sms_engagespark;

use Drupal\Core\Url;
use Drupal\sms_engagespark\Utility\PhoneNumberTrait;
use Drupal\sms_engagespark\Utility\SmsClientTrait;

/**
 * Represents a SMS for engageSPARK SMS.
 */
final class Sms {

  use PhoneNumberTrait;
  use SmsClientTrait;

  /**
   * The SMS message.
   *
   * @var string
   */
  protected $message;

  /**
   * The SMS recipient.
   *
   * @var string
   */
  protected $recipient;

  /**
   * The SMS sender.
   *
   * @var string
   */
  protected $sender;

  /**
   * Set the SMS message.
   *
   * @param int $message
   *   The SMS message.
   *
   * @return $this
   *   Returns the called object for chaining.
   */
  public function setMessage(string $message) {
    $this->message = $message;
    return $this;
  }

  /**
   * Set the SMS recipient.
   *
   * @param int $recipient
   *   The SMS recipient.
   *
   * @return $this
   *   Returns the called object for chaining.
   */
  public function setRecipient(string $recipient): self {
    $this->recipient = $recipient;
    return $this;
  }

  /**
   * Set the SMS sender.
   *
   * @param int $sender
   *   The SMS sender.
   *
   * @return $this
   *   Returns the called object for chaining.
   */
  public function setSender(string $sender): self {
    $this->sender = $sender;
    return $this;
  }

  /**
   * Sends an SMS request and retrieves the response from the engageSPARK API.
   *
   * @see https://openapi.engagespark.com/
   *
   * @return array
   *   An array containing the API response data.
   *
   * @throws \Exception
   *   Throws an exception if the API request fails.
   */
  public function send() {
    // Retrieve the formatted phone number details for the recipient.
    /** @var \Drupal\sms_engagespark\Utility\PhoneNumberTrait[] $phone_instance */
    $phone_instance = $this->getPhoneInstance($this->recipient);

    // Prepare the request payload for the SMS API.
    $payload = [
      'to' => $phone_instance['number_formatted'],
      'message' => $this->message,
      'from' => $this->sender,
      'fullPhoneNumber' => $phone_instance['number_plain'],
      'phoneNumber' => $phone_instance['number_national'],
      'phoneNumberCountry' => $phone_instance['code_region'],
    ];

    // Send the SMS request using the API.
    $response = $this->call('POST', Url::fromUri('https://api.engagespark.com/v1/sms/phonenumber'), $payload);

    return $response;
  }

}
