<?php

namespace Drupal\sms_engagespark\Utility;

use libphonenumber\NumberParseException;
use libphonenumber\PhoneNumberFormat;
use libphonenumber\PhoneNumberToTimeZonesMapper;
use libphonenumber\PhoneNumberUtil;

/**
 * Provides a trait for the engageSPARK phone number.
 */
trait PhoneNumberTrait {

  /**
   * The utility for international phone numbers.
   *
   * @var \libphonenumber\PhoneNumberUtil
   */
  protected $instance;

  /**
   * An offline mapper from phone numbers to time zones.
   *
   * @var \libphonenumber\PhoneNumberToTimeZonesMapper
   */
  protected $timezoneMapperInstance;

  /**
   * Gets the international phone numbers utility.
   *
   * @return \libphonenumber\PhoneNumberUtil
   *   The utility for international phone numbers.
   */
  public function getPhoneNumberUtilInstance() {
    if (!$this->instance) {
      $this->instance = PhoneNumberUtil::getInstance();
    }
    return $this->instance;
  }

  /**
   * Gets the offline mapper from phone numbers.
   *
   * @return \libphonenumber\PhoneNumberToTimeZonesMapper
   *   The offline mapper from phone numbers to time zones.
   */
  public function getTimezoneMapperInstance() {
    if (!$this->timezoneMapperInstance) {
      $this->timezoneMapperInstance = PhoneNumberToTimeZonesMapper::getInstance();
    }
    return $this->timezoneMapperInstance;
  }

  /**
   * Parses a phone number and retrieves its details.
   *
   * @param string $number
   *   The phone number in E.164 format (e.g., +6390000000).
   *
   * @codingStandardsIgnoreStart
   *
   * @return array|null
   *   An associative array containing the parsed phone number details:
   *   - `code_country` (int): The country calling code (e.g., 63 for the Philippines).
   *   - `code_region` (string|null): The ISO region code (e.g., "PH" for the Philippines).
   *   - `number_formatted` (string): The full number in E.164 format without the "+" sign.
   *   - `number_plain` (string): The original input number.
   *   - `number_national` (int): The national format number without the country code.
   *   - `timezone` (string|null): The primary timezone for the phone number, if available.
   *   Returns NULL if parsing fails.
   *
   * @codingStandardsIgnoreEnd
   */
  public function getPhoneInstance(string $number) {
    // Get the phone number utility instance.
    /** @var \libphonenumber\PhoneNumberUtil $instance */
    $instance = $this->getPhoneNumberUtilInstance();

    try {
      // Parse the given phone number.
      $parse_number = $instance->parse($number);

      // Retrieve the associated time zones for the number.
      $timezone = $this->getTimezoneMapperInstance()->getTimeZonesForNumber($parse_number);

      // Format the number in E.164 format without the "+" sign.
      $formatted_number = ltrim($instance->format($parse_number, PhoneNumberFormat::E164), '+');

      return [
        'code_country' => $parse_number->getCountryCode(),
        'code_region' => $instance->getRegionCodeForNumber($parse_number),
        'number_formatted' => $formatted_number,
        'number_plain' => $number,
        'number_national' => $parse_number->getNationalNumber(),
        'timezone' => $timezone[0] ?? NULL,
      ];
    }
    catch (NumberParseException $e) {
      // Log the exception message if the number parsing fails.
      \Drupal::logger('sms_engagespark')->debug($e->getMessage());
    }
  }

}
