<?php

namespace Drupal\sms_engagespark\Utility;

use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\ClientException;
use Symfony\Component\HttpKernel\Exception\HttpException;

/**
 * Provides functionality for interacting with the engageSPARK service.
 */
trait SmsClientTrait {

  use MessengerTrait;
  use StringTranslationTrait;

  /**
   * The guzzle client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected ClientInterface $client;

  /**
   * The organization ID.
   *
   * @var string
   */

  protected string $orgId;

  /**
   * The usercode and password.
   *
   * @var array
   */
  protected array $credentials = [];

  /**
   * Set guzzle client interface.
   *
   * @param \GuzzleHttp\ClientInterface $client
   *   The guzzle client interface to make HTTP requests.
   *
   * @return $this
   *   Returns the called object for chaining.
   */
  public function setClient(ClientInterface $client): self {
    $this->client = $client;
    return $this;
  }

  /**
   * Set the organization ID.
   *
   * @param string $org_id
   *   Returns the organization ID.
   *
   * @return $this
   *   Returns the called object for chaining.
   */
  public function setOrgId(string $org_id): self {
    $this->orgId = $org_id;
    return $this;
  }

  /**
   * Set the token.
   *
   * @param array $credentials
   *   The token for HTTP request authentication.
   *
   * @example
   *   [
   *     'token' => 'token_value',
   *   ]
   *
   * @return $this
   *   Returns the called object for chaining.
   */
  public function setCredentials(array $credentials): self {
    $this->credentials = $credentials;
    return $this;
  }

  /**
   * Makes an HTTP request using Guzzle.
   *
   * @param string $method
   *   The HTTP method to be used for the request (e.g., GET, POST).
   * @param \Drupal\Core\Url $url
   *   The URL object representing the request endpoint.
   * @param array $payload
   *   The request payload containing necessary parameters.
   *
   * @return string|null
   *   The response content if successful, or NULL if an error occurs.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Thrown when the HTTP request is not successful.
   * @throws \GuzzleHttp\Exception\ClientException
   *   Thrown if the HTTP client encounters an error.
   */
  public function call(string $method, Url $url, array $payload) {
    $error_message = $this->t('SMS client encountered an error. Please try again later or contact the site administrator.');
    $content = '';

    // Append the organization ID to the request data.
    $payload['orgId'] = $this->orgId;

    // Attempt HTTP request with Guzzle, handle responses and exceptions.
    try {
      // Perform the HTTP request using Guzzle.
      $response = $this->client->request($method, $url->toString(), [
        'verify' => FALSE,
        'timeout' => 60,
        'headers' => [
          'Authorization' => $this->credentials['token'],
          'Content-Type' => 'application/json',
        ],
        'json' => $payload,
      ]);

      // Retrieve the response body content.
      $content = $response->getBody()->getContents();

      // If the status code is 200, throw an HTTP exception.
      if ($response->getStatusCode() !== 200) {
        throw new HttpException(400, $error_message);
      }

      // Display a message.
      $this->messenger()->addMessage($this->t('engageSPARK: SMS has been sent successfully to %phone_number.', [
        '%phone_number' => $this->data['to'],
      ]));

      return $content;
    }
    catch (ClientException | HttpException $e) {
      // Log a message.
      \Drupal::logger('sms_engagespark')->error($method . ': ' . $e->getMessage());
    }
  }

}
